"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useSubscribeToEditorView = useSubscribeToEditorView;
var _jotai = require("jotai");
var _react = _interopRequireDefault(require("react"));
var _editorState = require("../state/editor-state");
var _editorView = require("../state/editor-view");
var _history = require("../state/history");
var _subscribeOnUpdates = _interopRequireDefault(require("../utils/subscribe-on-updates"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function useSubscribeToEditorView(editorView, diffWorkerInstance) {
  var setEditorView = (0, _jotai.useSetAtom)(_editorView.editorViewAtom);
  var historyDispatcher = (0, _jotai.useSetAtom)(_history.historyWriteAtom);
  var setEditorState = (0, _jotai.useSetAtom)(_editorState.editorStateAtom);
  var diffWorker = diffWorkerInstance ? Promise.resolve().then(function () {
    return _interopRequireWildcard(require("../state/json-diff-worker"));
  }).then(function (_ref) {
    var JsonDiffWorker = _ref.JsonDiffWorker;
    return new JsonDiffWorker(diffWorkerInstance);
  }) : Promise.resolve().then(function () {
    return _interopRequireWildcard(require("../state/json-diff-main"));
  }).then(function (_ref2) {
    var JsonDiffMain = _ref2.JsonDiffMain;
    return new JsonDiffMain();
  });
  _react["default"].useEffect(function () {
    // set initial editor state
    setEditorState(editorView.state);

    // store editor view reference
    setEditorView(editorView);
    historyDispatcher({
      type: "reset",
      payload: {
        state: editorView.state
      }
    });
    (0, _subscribeOnUpdates["default"])(editorView, function (tr, oldState, newState) {
      setEditorState(newState);
      historyDispatcher({
        type: "update",
        payload: {
          oldState: oldState,
          newState: newState,
          tr: tr,
          diffWorker: diffWorker
        }
      });
    });
  }, [editorView, diffWorker]);
}